local AceGUI = LibStub("AceGUI-3.0")

local editbox = nil;
local btnImport = nil;

local enum = {
	INVITES = 1,
	HISTDATE = 2,
	HISTINST = 3,
	HISTBOSS = 4,
};
local windowType;

-- History toggle action
-- =========================
function MorthalinRaidTools:HideEditWindow()
	if (MorthalinRaidTools.ui.editFrame:IsVisible()) then
		MorthalinRaidTools.ui.editFrame:Hide();
	end
end

function MorthalinRaidTools:ShowEditWindow(import, winType, data)
	windowType = winType;
	
	if (MorthalinRaidTools.ui.editFrame:IsVisible() == false) then
		MorthalinRaidTools.ui.editFrame:Show();
	end
	
	if(import) then
		MorthalinRaidTools.ui.editFrame:SetHeight(500);
		btnImport.frame:Show();
	else
		MorthalinRaidTools.ui.editFrame:SetHeight(475);
		btnImport.frame:Hide();
	end
	
	if(windowType == enum.HISTBOSS) then
		MorthalinRaidTools:CreateCSVBossBuffs(data);
	elseif(windowType == enum.HISTINST) then
		MorthalinRaidTools:CreateCSVInstancePlayers(data);
	elseif(windowType == enum.HISTDATE) then
		MorthalinRaidTools:CreateCSVDatePlayers(data);
	end
end

function MorthalinRaidTools:ConstructEditFrame()
	MorthalinRaidTools.ui.editFrame = AceGUI:Create("Frame");
	MorthalinRaidTools.ui.editFrame:SetTitle("Import/Export");
	MorthalinRaidTools.ui.editFrame:SetStatusText("");
	MorthalinRaidTools.ui.editFrame:SetCallback("OnClose", function(widget) --[[AceGUI:Release(widget)--]] end);
	MorthalinRaidTools.ui.editFrame:SetLayout("Flow");
	MorthalinRaidTools.ui.editFrame:SetWidth(525);
	MorthalinRaidTools.ui.editFrame:SetHeight(475);
	MorthalinRaidTools.ui.editFrame:EnableResize(false);
	
	editbox = AceGUI:Create("MultiLineEditBox");
	editbox:SetLabel("");
	editbox:DisableButton(true);
	editbox:SetWidth(500);
	editbox:SetHeight(400);
	MorthalinRaidTools.ui.editFrame:AddChild(editbox);
	
	btnImport = AceGUI:Create("Button");
	btnImport:SetText("Import");
	btnImport:SetCallback("OnClick", function() self:Import(); end);
	MorthalinRaidTools.ui.editFrame:AddChild(btnImport);
	
	MorthalinRaidTools.ui.editFrame:Hide();
end

function MorthalinRaidTools:Import()
	MorthalinRaidTools:Print(tostring(windowType));
end

function MorthalinRaidTools:CreateCSVBossBuffs(kill)
	local buffsSorted = self.db.global.buffs;
	local text = kill.bossName .. " " .. date("%d.%m.%y %H:%M:%S",kill.killDate);
	
	local function sortBuffs(a, b)
		return a.name < b.name;
	end
	table.sort(buffsSorted, sortBuffs);
	
	for i = 1, #buffsSorted do
		text = text .. ";" .. buffsSorted[i].name;
	end
	text = text .. "\r\n";
	
	for i = 1, #kill.members do
		text = text .. kill.members[i].name;
		if(kill.members[i].group == 0) then
			text = text .. "(B)";
		end
		local playerSorted =  {};
		if(kill.members[i].buffs ~= nil) then
			for name, value in pairs(kill.members[i].buffs) do
				table.insert(playerSorted,{name = name, data = value});
			end
			table.sort(playerSorted, sortBuffs);
		end
		
		for j = 1, #buffsSorted do
			text = text .. ";";
			if(kill.members[i].group ~= 0 and MorthalinRaidTools:CheckBuffOnPlayer(buffsSorted[j], playerSorted, kill.members[i].class, kill.members[i].name)) then
				text = text .. "1";
			elseif(kill.members[i].group ~= 0) then
				text = text .. "0";
			end
		end
		text = text .. "\r\n";
	end
	
	editbox:SetText(text);
	editbox:HighlightText(0, #text);
	editbox:SetFocus();
end

function MorthalinRaidTools:CheckBuffOnPlayer(setting, buffs, class, name)
	local ret = false;
	for i = 1, #buffs do
		if(setting.name:gsub("%s+", "") == buffs[i].name:gsub("%s+", "")) then
			for j = 1, #setting.classes do
				if(class == setting.classes[j]) then
					ret = true;
					j = #buffs[i];
				end
			end
			if(ret == false) then
				for j = 1, #setting.include do
					if(name == setting.include[j]) then
						ret = true;
					end
				end
			end
			if(ret == true) then
				for j = 1, #setting.exclude do
					if(name == setting.exclude[j]) then
						ret = false;
					end
				end
			end
			i = #buffs;
		end
	end
	return ret;
end

function MorthalinRaidTools:CreateCSVInstancePlayers(instance)
	local rows = self.db.global.history;
	local kills = {};
	local players = {};
	local alts = {};
	local text = instance.name;
	
	for i=1, GetNumGuildMembers() do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if(string.sub(officernote, 1, 4) == "Alt:") then
			if(alts[string.sub(name, 1, name:find("-") - 1)] == nil) then
				alts[string.sub(name, 1, name:find("-") - 1)] = string.sub(officernote, 5, -1);
			end
		end
	end
	
	for i = 1, #rows do
		for j = 1, #self.db.global.history[i].members do
			local tempName = {};
			if(alts[self.db.global.history[i].members[j].name] ~= nil) then
				tempName = alts[self.db.global.history[i].members[j].name];
			else
				tempName = self.db.global.history[i].members[j].name;
			end
			if(players[tempName] == nil) then
				players[tempName] = {name = tempName};
			end
		end
		
		if (date("%H",rows[i].killDate) == "00" or date("%H",rows[i].killDate) == "01") then
			dateTemp = date("%d.%m.%Y",(rows[i].killDate - 7200))
		else
			dateTemp = date("%d.%m.%Y",rows[i].killDate)
		end
		if (dateTemp == date("%d.%m.%Y",instance.start) and rows[i].zone == instance.name and rows[i].instanceID == instance.instanceID) then
			table.insert(kills, rows[i]);
		end
	end	
	
	local function sortKills(a, b)
		return a.killDate < b.killDate;
	end
	table.sort(kills, sortKills);
	
	for i = 1, #kills do
		text = text .. ";" .. kills[i].bossName;
	end
	text = text .. ";All\r\n";
	
	players = self:ConvertStringIndex(players);
	local function sortPlayers(a, b)
		return a.index < b.index;
	end
	table.sort(players, sortPlayers);
	
	for i = 1, #players do
		local count = 0;
		text = text .. players[i].name;
		for j = 1, #kills do
			text = text .. ";";
			local found = false;
			for k = 1, #kills[j].members do
				if(kills[j].members[k].name  == players[i].name) then
					if(kills[j].members[k].group == 0) then
						text = text .. "B";
					else
						text = text .. "1";
					end
					count = count + 1;
					found = true;
					k = #kills[j].members;
				end
			end
			if(found == false) then
				text = text .. "0";
			end
		end
		text = text .. ";" .. string.gsub(tostring(self:Round((count / #kills), 2)), "%.", ",") .. "\r\n";
	end
	
	editbox:SetText(text);
	editbox:HighlightText(0, #text);
	editbox:SetFocus();
end

function MorthalinRaidTools:CreateCSVDatePlayers(raidDate)
	local rows = self.db.global.history;
	local kills = {};
	local players = {};
	local alts = {};
	local text = raidDate;
	local raids = {};
	
	for i=1, GetNumGuildMembers() do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if(string.sub(officernote, 1, 4) == "Alt:") then
			if(alts[string.sub(name, 1, name:find("-") - 1)] == nil) then
				alts[string.sub(name, 1, name:find("-") - 1)] = string.sub(officernote, 5, -1);
			end
		end
	end
	
	for i = 1, #rows do
		for j = 1, #self.db.global.history[i].members do
			local tempName = {};
			if(alts[self.db.global.history[i].members[j].name] ~= nil) then
				tempName = alts[self.db.global.history[i].members[j].name];
			else
				tempName = self.db.global.history[i].members[j].name;
			end
			if(players[tempName] == nil) then
				players[tempName] = {name = tempName, counts = {}};
			end
		end
		
		if (date("%H",rows[i].killDate) == "00" or date("%H",rows[i].killDate) == "01") then
			dateTemp = date("%d.%m.%Y",(rows[i].killDate - 7200))
		else
			dateTemp = date("%d.%m.%Y",rows[i].killDate)
		end
		if (dateTemp == raidDate) then
			table.insert(kills, rows[i]);
			
			if(raids[rows[i].zone] == nil) then
				raids[rows[i].zone] = 1;
			else
				raids[rows[i].zone] = raids[rows[i].zone] + 1;
			end
		end
	end	
	
	local function sortKills(a, b)
		return a.killDate < b.killDate;
	end
	table.sort(kills, sortKills);
	
	for i = 1, #kills do
		text = text .. ";" .. kills[i].bossName;
	end
	
	for name, count in pairs(raids) do
		text = text .. ";" .. name;
	end
	text = text .. ";All\r\n";
	
	players = self:ConvertStringIndex(players);
	local function sortPlayers(a, b)
		return a.index < b.index;
	end
	table.sort(players, sortPlayers);
	
	for i = 1, #players do
		text = text .. players[i].name;
		for j = 1, #kills do
			text = text .. ";";
			local found = false;
			for k = 1, #kills[j].members do
				if(kills[j].members[k].name  == players[i].name) then
					if(kills[j].members[k].group == 0) then
						text = text .. "B";
					else
						text = text .. "1";
					end
					if(players[i].counts[kills[j].zone] == nil) then
						players[i].counts[kills[j].zone] = 1;
					else
						players[i].counts[kills[j].zone] = players[i].counts[kills[j].zone] + 1;
					end
					found = true;
					k = #kills[j].members;
				end
			end
			if(found == false) then
				text = text .. "0";
			end
		end
		local count, countAll = 0, 0;
		for name, counts in pairs(raids) do
			count = count + (players[i].counts[name] == nil and 0 or players[i].counts[name]);
			countAll = countAll + counts;
			text = text .. ";" .. string.gsub(tostring(self:Round((players[i].counts[name] == nil and 0 or (players[i].counts[name] / counts)), 2)), "%.", ",");
		end
		text = text .. ";" .. string.gsub(tostring(self:Round((count / countAll), 2)), "%.", ",") .. "\r\n";
	end
	
	editbox:SetText(text);
	editbox:HighlightText(0, #text);
	editbox:SetFocus();
end
