local AceGUI = LibStub("AceGUI-3.0")

local zonePick, bossPick, groupPick, btnCreateRaid;

local killTimeText;
local historyMembers = {};

function MorthalinRaidTools:ConstructHistoryCreation()
	self.ui.hCreateFrame = AceGUI:Create("Frame");
	self.ui.hCreateFrame:SetTitle("History Creation");
	self.ui.hCreateFrame:SetStatusText("");
	self.ui.hCreateFrame:SetLayout("Flow");
	self.ui.hCreateFrame:SetWidth(445);
	self.ui.hCreateFrame:SetHeight(210);
	self.ui.hCreateFrame:EnableResize(false);
	self.ui.hCreateFrame.frame:EnableMouse(true);
	self.ui.hCreateFrame.frame:SetUserPlaced(true);
	
	zonePick = AceGUI:Create("Dropdown");
	zonePick:SetLabel("Zone");
	zonePick:SetWidth(200);
	zonePick:SetCallback("OnValueChanged", function() MorthalinRaidTools:GetAllBosses(); end);
	self.ui.hCreateFrame:AddChild(zonePick);
	MorthalinRaidTools:GetAllZones();
	
	bossPick = AceGUI:Create("Dropdown");
	bossPick:SetLabel("Boss");
	bossPick:SetWidth(200);
	bossPick:SetDisabled(true);
	bossPick:SetCallback("OnValueChanged", function() 
		if(MorthalinRaidTools:CheckAllFilled()) then 
			btnCreateRaid:SetDisabled(false); 
		else 
			btnCreateRaid:SetDisabled(true); 
		end 
	end);
	self.ui.hCreateFrame:AddChild(bossPick);
	
	groupPick = AceGUI:Create("Dropdown");
	groupPick:SetLabel("Copy groups from:");
	groupPick:SetWidth(400);
	groupPick:SetCallback("OnEnter", function() MorthalinRaidTools:ShowGroupMembers(); end);
	groupPick:SetCallback("OnLeave", function() MorthalinRaidTools:HideTooltip(); end);
	self.ui.hCreateFrame:AddChild(groupPick);
	MorthalinRaidTools:GetAllGroups();
	
	killTime = AceGUI:Create("EditBox");
	killTime:SetLabel("Kill time");
	killTime:SetWidth(150);
	killTime:SetCallback("OnEnterPressed", function(widget, event, text) 
		if(self:CheckDateTimeString(text) == false) then 
			GameTooltip:SetOwner(killTime.frame,"ANCHOR_RIGHT");
			GameTooltip:AddLine("Wrong DateTime Format (dd.mm.yyyy hh:mm:ss)");
			GameTooltip:Show();
		else
			GameTooltip:Hide();
			killTimeText = text;
			if(MorthalinRaidTools:CheckAllFilled()) then 
				btnCreateRaid:SetDisabled(false); 
			else 
				btnCreateRaid:SetDisabled(true); 
			end
		end
	end);
	self.ui.hCreateFrame:AddChild(killTime);
	killTime:SetText(date("%d.%m.%Y %H:%M:%S", GetServerTime()));
	killTimeText = date("%d.%m.%Y %H:%M:%S", GetServerTime());
	
	btnCreateRaid = AceGUI:Create("Button");
	btnCreateRaid:SetText("Create raid");
	btnCreateRaid:SetCallback("OnClick", function() MorthalinRaidTools:CreateManualKills(); end)
	btnCreateRaid:SetDisabled(true);
	self.ui.hCreateFrame:AddChild(btnCreateRaid);
	
	--self.ui.hCreateFrame:Hide();
end

function MorthalinRaidTools:ShowHistoryCreation()
	if(self.ui.hCreateFrame:IsVisible() == false) then
		self.ui.hCreateFrame:Show();
	end
end

function MorthalinRaidTools:HideHistoryCreation()
	if(self.ui.hCreateFrame:IsVisible()) then
		self.ui.hCreateFrame:Hide();
	end
end

function MorthalinRaidTools:CheckDateTimeString(text)
	local day, month, year, hour, minute, second = text:match('(%d+).(%d+).(%d+) (%d+):(%d+):(%d+)');
	if(day == nil or month == nil  or year == nil  or hour == nil  or minute == nil  or second == nil) then
		return false;
	end
	if(string.len(day) ~= 2) then
		day = "0" .. day;
	end
	if(string.len(month) ~= 2) then
		month = "0" .. month;
	end
	if(string.len(hour) ~= 2) then
		hour = "0" .. hour;
	end
	if(string.len(minute) ~= 2) then
		minute = "0" .. minute;
	end
	if(string.len(second) ~= 2) then
		second = "0" .. second;
	end
	text = day .. "." .. month .. "." .. year .. " " .. hour .. ":" .. minute .. ":" .. second;
	local dateInt = time{year=year, month=month, day=day, hour=hour, min=minute, sec=second};
	return text == date("%d.%m.%Y %H:%M:%S", dateInt);
end

function MorthalinRaidTools:GetAllZones()
	local tempInstances = {};
	local index = 1;
	zonePick:SetList(nil);
	for ind, data in pairs(self.db.global.encounterList) do
		if(tempInstances[data.instance] == nil) then
			tempInstances[data.instance] = index;
			zonePick:AddItem(index, data.instance);
			index = index + 1;
		end
	end
	zonePick:AddItem(index, nil);
	zonePick:SetValue(nil);
end

function MorthalinRaidTools:GetAllBosses()
	if(zonePick.text:GetText() ~= nil) then
		local tempInstances = {};
		local index = 1;
		bossPick:SetList(nil);
		for ind, data in pairs(self.db.global.encounterList) do
			if(data.instance == zonePick.text:GetText() and tempInstances[data.name] == nil) then
				tempInstances[data.name] = index;
				bossPick:AddItem(index, data.name);
				index = index + 1;
			end
		end
		bossPick:AddItem(index, "All");
		bossPick:SetDisabled(false);
	else
		bossPick:SetDisabled(true);
	end
end

function MorthalinRaidTools:CheckAllFilled()
	return (self:CheckDateTimeString(killTimeText) and zonePick.text:GetText() ~= nil and bossPick.text:GetText() ~= nil);
end

function MorthalinRaidTools:GetAllGroups()
	local sortedHistory = MorthalinRaidTools.db.global.history;
	local function sortKills(a, b)
		return a.killDate > b.killDate;
	end
	table.sort(sortedHistory, sortKills);
	groupPick:SetList(nil);
	groupPick:AddItem(1, "none");
	for i = 1, #sortedHistory do
		groupPick:AddItem(1 + i, date("%d.%m.%Y %H:%M:%S", sortedHistory[i].killDate) .. ": " .. sortedHistory[i].zone .. " - " .. sortedHistory[i].bossName);
		historyMembers[i + 1] = sortedHistory[i].members;
	end
	groupPick:SetValue(1);
end

function MorthalinRaidTools:ShowGroupMembers()
	if(groupPick:GetValue() ~= 1) then
		GameTooltip:SetOwner(groupPick.frame,"ANCHOR_RIGHT");
		GameTooltip:AddLine("Players");
		for i = 1, #historyMembers[groupPick:GetValue()] do
			GameTooltip:AddLine("|cff" .. historyMembers[groupPick:GetValue()][i].color .. historyMembers[groupPick:GetValue()][i].name);
		end
		GameTooltip:Show();
	end
end

function MorthalinRaidTools:CreateManualKills()
	if(bossPick.text:GetText() == "All") then
		local tempInstances = {};
		for ind, data in pairs(self.db.global.encounterList) do
			if(data.instance == zonePick.text:GetText() and tempInstances[data.name] == nil) then
				tempInstances[data.name] = 1;
				MorthalinRaidTools:CreateManualKill(data.name);
			end
		end
	else
		MorthalinRaidTools:CreateManualKill(bossPick.text:GetText())
	end
end

function MorthalinRaidTools:CreateManualKill(bossName)
	local day, month, year, hour, minute, second = killTimeText:match('(%d+).(%d+).(%d+) (%d+):(%d+):(%d+)');
	local dateInt = time{year=year, month=month, day=day, hour=hour, min=minute, sec=second};
	local timesheet = {
		bossID = self.db.global.encounterList[bossName].id,
		bossName = bossName,
		killDate = dateInt,
		zone = zonePick.text:GetText(),
		instanceID = tonumber(date("%d%m%y",GetServerTime()) .. zonePick:GetValue()),
		resetDate = MorthalinRaidTools:GetResetDate(dateInt, zonePick.text:GetText()),
		members = {},
	}
	timesheet.members = historyMembers[groupPick:GetValue()];
	
	table.insert(self.db.global.history, timesheet);
end

function MorthalinRaidTools:GetResetDate(killDate, instance)
	local looping = true;
	local reset;
	local step;
	local up = true;
	if(instance == "Molten Core" or instance == "Blackwing Lair" or instance == "Ahn'Qiraj Temple" or instance == "Naxxramas") then
		reset = MorthalinRaidTools.db.global.resets["MC"];
		step = 604800;
	elseif(instance == "Onyxia's Lair") then
		reset = MorthalinRaidTools.db.global.resets["Onyx"];
		step = 432000;
	elseif(instance == "Zul'Gurub" or instance == "Ruins of Ahn'Qiraj") then
		reset = MorthalinRaidTools.db.global.resets["ZG"];
		step = 259200;
	end
	
	if(reset > killDate) then
		up = false;
	end
	
	while looping do
		if(up) then
			if(killDate < reset) then
				looping = false;
			else
				reset = reset + step;
			end
		else
			if(killDate > reset) then
				looping = false;
			else
				reset = reset - step;
			end
		end
	end
	return date("%d.%m.%Y", reset);
end
